#include <UDP_Protocol.h>

using namespace inaf::oasbo::ConnectionProtocols;

UDPProtocolBuilder::UDPProtocolBuilder() {
	this->reset();
}
UDPProtocolBuilder::UDPProtocolBuilder(std::string ip, int port) {
	this->protocol = new UDPProtocol(ip, port);

}
UDPProtocolBuilder::~UDPProtocolBuilder() {
	delete protocol;
}

void UDPProtocolBuilder::reset() {
	this->protocol = new UDPProtocol();
}

UDPProtocolBuilder* UDPProtocolBuilder::configFrom(Configurators::BaseConfigurator &conf) {
	conf.readConfigFromSource(config_target);
	std::map<std::string, std::string> params = conf.getConfig();

	std::string key = config_target+"_"+ip_key;
	if (params.count(key) > 0)
		protocol->setIp(params[key]);

	key = config_target+"_"+port_key;
	if (params.count(key) > 0)
		protocol->setPort(std::stoi(params[key]));
	return this;
}

UDPProtocolBuilder* UDPProtocolBuilder::setIp(std::string ip) {
	protocol->setIp(ip);
	return this;
}

UDPProtocolBuilder* UDPProtocolBuilder::setPort(int port) {
	protocol->setPort(port);
	return this;
}

UDPProtocol* UDPProtocolBuilder::getProtocol() {
	UDPProtocol *result = this->protocol;
	this->reset();
	return result;
}
